import json
import os
from .dict import ToolConfDict


class ToolConfJsonFile(ToolConfDict):
    _configs_dir = None

    def __init__(self, config_file):
        if not os.path.isfile(config_file):
            raise Exception("LTI tool config file not found: " + config_file)
        self._configs_dir = os.path.dirname(config_file)

        cfg = open(config_file, 'r')
        iss_conf = json.loads(cfg.read())
        super(ToolConfJsonFile, self).__init__(iss_conf)
        cfg.close()

        for iss in iss_conf:
            private_key_file = iss_conf[iss]['private_key_file']
            if not private_key_file.startswith('/'):
                private_key_file = self._configs_dir + '/' + private_key_file

            prf = open(private_key_file, 'r')
            self.set_private_key(iss, prf.read())
            prf.close()



from pylti1p3.session import SessionService


class DjangoSessionService(SessionService):
    _request = None

    def __init__(self, request):
        self._request = request

    def _get_key(self, key, nonce=None):
        return self._session_prefix + '-' + key + (('-' + nonce) if nonce else '')

    def get_launch_data(self, key):
        return self._get_value(self._get_key(key))

    def save_launch_data(self, key, jwt_body):
        self._set_value(self._get_key(key), jwt_body)

    def save_nonce(self, nonce):
        self._set_value(self._get_key('nonce', nonce), True)

    def check_nonce(self, nonce):
        nonce_key = self._get_key('nonce', nonce)
        return nonce_key in self._request.session

    def save_state_params(self, state, params):
        self._set_value(self._get_key(state), params)

    def get_state_params(self, state):
        return self._get_value(self._get_key(state))

    def _set_value(self, key, value):
        self._request.session[key] = value

    def _get_value(self, key):
        return self._request.session.get(key, None)



from .abstract import ToolConfAbstract
from ..registration import Registration
from ..deployment import Deployment


class ToolConfDict(ToolConfAbstract):
    _config = None
    _private_key = {}

    def __init__(self, json_data):
        super(ToolConfDict, self).__init__()
        self._config = json_data

    def find_registration_by_issuer(self, iss, *args, **kwargs):
        if iss not in self._config:
            raise Exception('iss %s not found in settings' % iss)

        iss_conf = self._config[iss]

        reg = Registration()
        return reg.set_auth_login_url(iss_conf['auth_login_url'])\
            .set_auth_token_url(iss_conf['auth_token_url'])\
            .set_client_id(iss_conf['client_id'])\
            .set_key_set(iss_conf['key_set'])\
            .set_key_set_url(iss_conf['key_set_url'])\
            .set_issuer(iss)\
            .set_tool_private_key(self.get_private_key(iss))

    def find_deployment(self, iss, deployment_id):
        if iss not in self._config:
            raise Exception('iss %s not found in settings' % iss)
        if deployment_id not in self._config[iss]['deployment_ids']:
            return None

        d = Deployment()
        return d.set_deployment_id(deployment_id)

    def set_private_key(self, iss, key_content):
        self._private_key[iss] = key_content

    def get_private_key(self, iss):
        return self._private_key[iss]



import re


class NamesRolesProvisioningService(object):
    _service_connector = None
    _service_data = None

    def __init__(self, service_connector, service_data):
        self._service_connector = service_connector
        self._service_data = service_data

    def get_members(self):
        members = []
        next_page = self._service_data['context_memberships_url']

        while next_page:
            page = self._service_connector.make_service_request(
                ['https://purl.imsglobal.org/spec/lti-nrps/scope/contextmembership.readonly'],
                next_page,
                accept='application/vnd.ims.lti-nrps.v2.membershipcontainer+json'
            )

            members.extend(page.get('body', {}).get('members', []))

            next_page = False
            link_header = page.get('headers', {}).get('link', '')
            if link_header:
                match = re.search(r'<(.*)>; ?rel="next"', link_header.lower().strip())
                if match:
                    next_page = match.group(1)

        return members



import json


class Grade(object):
    _score_given = None
    _score_maximum = None
    _activity_progress = None
    _grading_progress = None
    _timestamp = None
    _user_id = None

    def get_score_given(self):
        return self._score_given

    def set_score_given(self, value):
        self._score_given = value
        return self

    def get_score_maximum(self):
        return self._score_maximum

    def set_score_maximum(self, value):
        self._score_maximum = value
        return self

    def get_activity_progress(self):
        return self._activity_progress

    def set_activity_progress(self, value):
        self._activity_progress = value
        return self

    def get_grading_progress(self):
        return self._grading_progress

    def set_grading_progress(self, value):
        self._grading_progress = value
        return self

    def get_timestamp(self):
        return self._timestamp

    def set_timestamp(self, value):
        self._timestamp = value
        return self

    def get_user_id(self):
        return self._user_id

    def set_user_id(self, value):
        self._user_id = value

    def get_value(self):
        data = {
            'scoreGiven': self._score_given if self._score_given else None,
            'scoreMaximum': self._score_maximum if self._score_maximum else None,
            'activityProgress': self._activity_progress,
            'gradingProgress': self._grading_progress,
            'timestamp': self._timestamp,
            'userId': self._user_id
        }
        return json.dumps({k: v for k, v in data.items() if v is not None})



import werkzeug

from pylti1p3.cookie import CookieService


class FlaskCookieService(CookieService):
    _request = None
    _cookie_data_to_set = None

    def __init__(self, request):
        self._request = request
        self._cookie_data_to_set = {}

    def _get_key(self, key):
        return self._cookie_prefix + '-' + key

    def get_cookie(self, name):
        return self._request.get_cookie(self._get_key(name))

    def set_cookie(self, name, value, exp=3600):
        self._cookie_data_to_set = {
            'key': self._get_key(name),
            'value': value,
            'exp': exp
        }

    def update_response(self, response):
        if self._cookie_data_to_set:
            cookie_kwargs = dict(
                key=self._cookie_data_to_set['key'],
                value=self._cookie_data_to_set['value'],
                max_age=self._cookie_data_to_set['exp'],
                secure=self._request.is_secure(),
                path='/',
                httponly=True,
            )

            werkzeug_version = int(werkzeug.__version__.split('.')[0])
            if self._request.is_secure() and werkzeug_version >= 1:
                cookie_kwargs['samesite'] = 'None'

            response.set_cookie(**cookie_kwargs)



from pylti1p3.session import SessionService


class FlaskSessionService(SessionService):
    _request = None

    def __init__(self, request):
        self._request = request

    def _get_key(self, key, nonce=None):
        return self._session_prefix + '-' + key + \
               (('-' + nonce) if nonce else '')

    def get_launch_data(self, key):
        return self._request.session.get(self._get_key(key), None)

    def save_launch_data(self, key, jwt_body):
        self._request.session[self._get_key(key)] = jwt_body

    def save_nonce(self, nonce):
        self._request.session[self._get_key('nonce', nonce)] = True

    def check_nonce(self, nonce):
        return self._get_key('nonce', nonce) in self._request.session

    def save_state_params(self, state, params):
        self._request.session[self._get_key(state)] = params

    def get_state_params(self, state):
        return self._request.session[self._get_key(state)]



class Deployment(object):

    _deployment_id = None

    def get_deployment_id(self):
        return self._deployment_id

    def set_deployment_id(self, deployment_id):
        self._deployment_id = deployment_id
        return self



import warnings
from abc import ABCMeta, abstractmethod


class ToolConfAbstract(object):
    __metaclass__ = ABCMeta
    reg_extended_search = False

    def __init__(self):
        co_varnames = self.find_registration_by_issuer.__code__.co_varnames
        self.reg_extended_search = all([arg in co_varnames for arg in ('args', 'kwargs')])

    def find_registration(self, iss, *args, **kwargs):
        if self.reg_extended_search:
            return self.find_registration_by_issuer(iss, *args, **kwargs)
        else:
            warnings.warn("Signature of ToolConfAbstract.find_registration_by_issuer method was changed, "
                          "please update you custom implementation",
                          DeprecationWarning)
            return self.find_registration_by_issuer(iss)

    @abstractmethod
    def find_registration_by_issuer(self, iss, *args, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def find_deployment(self, iss, deployment_id):
        raise NotImplementedError



class Registration(object):
    _issuer = None
    _client_id = None
    _key_set_url = None
    _key_set = None
    _auth_token_url = None
    _auth_login_url = None
    _tool_private_key = None

    def get_issuer(self):
        return self._issuer

    def set_issuer(self, issuer):
        self._issuer = issuer
        return self

    def get_client_id(self):
        return self._client_id

    def set_client_id(self, client_id):
        self._client_id = client_id
        return self

    def get_key_set(self):
        return self._key_set

    def set_key_set(self, key_set):
        self._key_set = key_set
        return self

    def get_key_set_url(self):
        return self._key_set_url

    def set_key_set_url(self, key_set_url):
        self._key_set_url = key_set_url
        return self

    def get_auth_token_url(self):
        return self._auth_token_url

    def set_auth_token_url(self, auth_token_url):
        self._auth_token_url = auth_token_url
        return self

    def get_auth_login_url(self):
        return self._auth_login_url

    def set_auth_login_url(self, auth_login_url):
        self._auth_login_url = auth_login_url
        return self

    def get_tool_private_key(self):
        return self._tool_private_key

    def set_tool_private_key(self, tool_private_key):
        self._tool_private_key = tool_private_key
        return self



from abc import ABCMeta, abstractmethod


class SessionService(object):
    __metaclass__ = ABCMeta
    _session_prefix = 'lti1p3'

    @abstractmethod
    def get_launch_data(self, key):
        raise NotImplementedError

    @abstractmethod
    def save_launch_data(self, key, jwt_body):
        raise NotImplementedError

    @abstractmethod
    def save_nonce(self, nonce):
        raise NotImplementedError

    @abstractmethod
    def check_nonce(self, nonce):
        raise NotImplementedError

    @abstractmethod
    def save_state_params(self, state, params):
        raise NotImplementedError

    @abstractmethod
    def get_state_params(self, state):
        raise NotImplementedError
